<?php namespace Service;

use Dompdf\Dompdf;
use Dompdf\Options;
use Service\Contracts\Generator;
use iio\libmergepdf\Merger;
use Exception;


class PdfGenerator implements Generator{
  
  private $options;
  private $dompdf ;
  private $dompdfOptions;
  private $data;
  private $content;

  private $resources;
  private $chroot;

  private $empty_page;
  
  protected static $instance = null;


  private $mergedFile;

  /**
   * get Instance
   *
   * @param $opts
   * @return PdfGenerator
  */
  public static function getInstance($opts){
    if( self::$instance == null ){
      $instance = new PdfGenerator($opts);
      self::$instance = $instance;
    }
    return self::$instance;
  }

  public function __construct($ops) {
    $this->empty_page = __DIR__.DIRECTORY_SEPARATOR.'Utils'.DIRECTORY_SEPARATOR.'empty.pdf';
    $this->options = $ops;
    $this->dompdfOptions = new Options();
    $this->data = array();


    if(isset($ops["rootPath"])) {
      if(file_exists($ops["rootPath"])) {
        //throw new Exception("Option 'rootPath' not found!");
        $this->chroot = $ops["rootPath"];
        $chroot = $ops["rootPath"];
        $rootDir = $ops["rootPath"];
        $tempDir = $ops["rootPath"] . DIRECTORY_SEPARATOR . 'lib' . DIRECTORY_SEPARATOR . 'temp';
        $fontDir = $ops["rootPath"] . DIRECTORY_SEPARATOR . 'lib' . DIRECTORY_SEPARATOR . 'fonts';
        $fontCache = $ops["rootPath"] . DIRECTORY_SEPARATOR . 'lib' . DIRECTORY_SEPARATOR . 'fonts' . DIRECTORY_SEPARATOR . 'cache';
    
        $this->dompdfOptions->setChroot($chroot);
        $this->dompdfOptions->setRootDir($rootDir);
        $this->dompdfOptions->setTempDir($tempDir);
        $this->dompdfOptions->setFontDir($fontDir);
        $this->dompdfOptions->setFontCache($fontCache);
      }
    }
    
    $this->dompdfOptions->set('isFontSubsettingEnabled', true);
    $this->dompdfOptions->set('enable_remote', true);
    $this->dompdfOptions->isHtml5ParserEnabled(true);
    $this->dompdfOptions->isRemoteEnabled(true);
    $this->dompdfOptions->isPhpEnabled(true);

    //if(isset($ops["defaultFont"]))
    //  $this->dompdfOptions->set('defaultFont', $ops["defaultFont"]);

    if(isset($ops["set_time_limit"]))
      set_time_limit($ops["set_time_limit"]);
    
    //if(isset($ops["nativeFonts"]))
    //  Dompdf::$nativeFonts = $ops["nativeFonts"];

    if(array_key_exists("attachment_separator",$ops)){
      $this->empty_page = $ops["attachment_separator"];
    }

    $this->InitDompdfInstance();

  }  


  private function InitDompdfInstance() {
    $this->dompdf = new Dompdf($this->dompdfOptions);
    if(isset($this->options["setPaper"]))
      $this->dompdf->setPaper($this->options["setPaper"][0], $this->options["setPaper"][1]);
  }

  /**
   * loadView
   *
   * @param $view
   * @return mixed
   */
  public function loadView($view) {
    ob_start();
    $i = extract($this->data);
    if(isset($this->content)) 
      extract($this->content);
    include($view);
    $buffer = ob_get_contents();
    @ob_end_clean();
    $this->dompdf->load_html($buffer);
  }

  /**
   * loadViewData
   *
   * @param $view
   * @return mixed
   */
  public function loadViewData($data) {
    $this->content = $data;
  }
  /**
   * loadViews
   *
   * @param $views
   * @return mixed
   */
  public function loadViews($views = array()){
    try{
      $buffer;
      $data = array();
      foreach($views as $key => $view){
        if(file_exists($view)){
          ob_start();
          $i = extract($this->data);
          if(isset($this->content)) 
            extract($this->content);
          if(count($data) > 0)
            extract($data);
          include($view);
          $buffer = ob_get_contents();
          @ob_end_clean();
    
          $data[$key] = $buffer;
        }else {
          $data[$key] = "";
        }
      }
  
      $this->dompdf->load_html($buffer);
    }catch(Exception $e){

    }
    
  }

  /**
   * generate
   *
   * @param $data
   * @param $views
   * @return Generator
   */
  public function generate($data, $views){
     try{
        $this->loadViewData($data);
        $this->loadViews($views);
        $dompdf = $this->DompdfInstance();
        $this->DompdfInstance()->render();
        return $this;
     }catch(Exception $e){
       return $this;
     }
      
  }

  /**
   * DompdfInstance
   *
   * Return the Dompdf instance in use
   *
   * @return Dompdf
   */
  public function DompdfInstance() {
    return $this->dompdf;
  }

  /**
   * mergeAttachmentsToPdf
   *
   * Return the Dompdf instance in use
   *
   * @param pdf
   * @param $attachments
   * @return mixed
   */
  public function mergeAttachmentsToPdf($pdf,$attachments){
    try{
      if(file_exists($mainFile)){
        $mainFile = file_get_contents($pdf);
        return $this->mergeAttachmentsToRawPdf($mainFile,$attachments);
      }
    }catch(Exception $e){
      //return "";
    }
    
  }

  /**
   * mergeAttachmentsToRawPdf
   *
   * Create a stream with the content of the $pdf + attachments
   *
   * @param pdf
   * @param $attachments
   * @return mixed
   */
  public function mergeAttachmentsToRawPdf($pdf,$attachments){
    
      $merger = new Merger();
      $tempDir = $this->options["rootPath"] . DIRECTORY_SEPARATOR . 'lib' . DIRECTORY_SEPARATOR . 'temp';
      if(file_exists($tempDir)){
        $merger->setTempDir($tempDir);
        $merger->addRaw($pdf);
        //check if there is a separator page
        $sep = "";
        if(count($attachments) > 1 && $this->empty_page != null && $this->empty_page != "" && file_exists($this->empty_page)){
          $sep = file_get_contents($this->empty_page);
        }
        //add the attachments as raw because of file pointers
        for($i=0 ; $i < count($attachments) ; $i++){
          try{
            if(file_exists($attachments[$i])){
              $content = file_get_contents($attachments[$i]);
              $merger->addRaw($content);
    
              if($i < count($attachments) - 1 && $sep != ""){
                $merger->addRaw($sep);
              }
            }
            
          }catch(Exception $e){
          }
        }
        
        try{
          $createdPdf = $merger->merge(true);
          
              return $createdPdf;
        }catch(Exception $e){
          
        }
       
      }
      return $pdf;
    
  }
}